/********************************************************************************
*
* MTRandClass - 'Mersenne Twister' random number generator. (mt19937)
*
* Copyright (C) 1997 Makoto Matsumoto and Takuji Nishimura. <matumoto@math.keio.ac.jp>
*  http://www.math.keio.ac.jp/~matumoto/emt.html
*
********************************************************************************/

#include "mtrand.h"


void MTRandClass::RandomSeed(unsigned long rand_seed)
{
	seed = rand_seed;
	
	unsigned long   x = (seed | 1U) & 0xFFFFFFFFU;
	unsigned long * s = state;
	long            j;

	for(left = 0, *(s++) = x, j = N; --j; *(s++) = (x *= 69069U) & 0xFFFFFFFFU);
}


unsigned long MTRandClass::ReloadMT(void)
{
	unsigned long * p0 = state;
	unsigned long * p2 = state + 2;
	unsigned long * pM = state + M;
	unsigned long   s0;
	unsigned long   s1;
	long            j;

	if(left < -1)
		RandomSeed(4357U);

	left = N - 1;
	next = state + 1;

	for(s0 = state[0], s1 = state[1], j = N - M + 1; --j; s0 = s1, s1 = *(p2++))
		*(p0++) = *(pM++) ^ (mixBits(s0, s1) >> 1) ^ (loBit(s1) ? K : 0U);

	for(pM = state, j = M; --j; s0 = s1, s1 = *(p2++))
		*(p0++) = *(pM++) ^ (mixBits(s0, s1) >> 1) ^ (loBit(s1) ? K : 0U);

	s1 = state[0];
	*p0 = *pM ^ (mixBits(s0, s1) >> 1) ^ (loBit(s1) ? K : 0U);
	
	s1 ^= (s1 >> 11);
	s1 ^= (s1 <<  7) & 0x9D2C5680U;
	s1 ^= (s1 << 15) & 0xEFC60000U;
	
	return(s1 ^ (s1 >> 18));
}
